//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
//  Explode plugin
//
//  Copyright (C)2008-2011 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

//
//    This is ECMAScript code (ECMA-262 aka "Java Script")
//

//---------------------------------------------------------
//    init
//    this function will be called on startup of
//    mscore
//---------------------------------------------------------

function init () {
}

// loop through music removing all but "nth" note from top of "nth" selected staff
function explode () {


  var cursor       = new Cursor(curScore);
  var selectionEnd = new Cursor(curScore);

  cursor.goToSelectionStart();
  selectionEnd.goToSelectionEnd();
  var startStaff = cursor.staff;
  var endStaff   = selectionEnd.staff;

  // must select at least two staves
  if (endStaff - startStaff < 2) {
    QMessageBox.warning(0,"Invalid Selection","Please select at least two staves and retry operation");
    return;
  }

  curScore.startUndo();

  for (var staff = startStaff; staff < endStaff; ++staff) {

    cursor.goToSelectionStart();
    cursor.voice = 0;
    cursor.staff = staff;

    while (cursor.tick() < selectionEnd.tick()) {

      var n = staff - startStaff;

      if (cursor.isChord()) {
        var chord = cursor.chord();
        var notes = chord.notes;
        if (notes > n) {
          for (var i = notes-1; i >= 0; --i) {
            if (i != (notes-n) - 1) {
              chord.removeNote(i);
            }
          }
        }
      }

      cursor.next();

    }

  }

  curScore.endUndo();

}

function run () {
  explode();
}

//---------------------------------------------------------
//    menu:  defines where the function will be placed
//           in the menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.Explode',
      init: init,
      run:  run
}

mscorePlugin;
