//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
//  Implode plugin
//
//  Copyright (C)2008-2011 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

//
//    This is ECMAScript code (ECMA-262 aka "Java Script")
//

//---------------------------------------------------------
//    init
//    this function will be called on startup of
//    mscore
//---------------------------------------------------------

function init () {
}

// loop through music
// add notes from subsequent selected staves to chords on first selected staff
// doesn't preserve ties
function implode () {

  var cursor       = new Cursor(curScore);
  var selectionEnd = new Cursor(curScore);

  cursor.goToSelectionStart();
  selectionEnd.goToSelectionEnd();

  var startStaff = cursor.staff;
  var endStaff   = selectionEnd.staff;

  // must select at least two staves
  if (endStaff - startStaff < 2) {
    QMessageBox.warning(0,"Invalid Selection","Please select at least two staves and retry operation");
    return;
  }

  // define array of cursors for source staves
  var subCursor = new Array();
  for (var staff = startStaff+1; staff < endStaff; ++staff) {
    subCursor[staff] = new Cursor(curScore);
    subCursor[staff].goToSelectionStart();
    subCursor[staff].voice = 0;
    subCursor[staff].staff = staff;
  }

  curScore.startUndo();

  cursor.goToSelectionStart();
  cursor.voice = 0;
  cursor.staff = startStaff;

  // loop through destination staff
  while (cursor.tick() < selectionEnd.tick()) {

    if (cursor.isChord()) {

      var chord = cursor.chord();

      for (var staff = startStaff+1; staff < endStaff; ++staff) {

        // advance to same tick position
        while (subCursor[staff].tick() < cursor.tick()) {
          subCursor[staff].next();
        }
        // if no note at this position, move on to next staff
        if ((subCursor[staff].tick() != cursor.tick()) || !subCursor[staff].isChord()) {
          continue;
        }

        var oldnote = subCursor[staff].chord().topNote();
        var newnote = new Note();
        newnote.pitch = oldnote.pitch;
        newnote.tpc = oldnote.tpc;
        newnote.userAccidental = oldnote.userAccidental;
        newnote.visible = oldnote.visible;
//        newnote.tied = oldnote.tied;
//        newnote.velocity = oldnote.velocity;
        chord.addNote(newnote);

      }

    }

    cursor.next();

  }

  curScore.endUndo();

}

function run () {
  implode();
}

//---------------------------------------------------------
//    menu:  defines where the function will be placed
//           in the menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.Implode',
      init: init,
      run:  run
}

mscorePlugin;
